@@ -37,6 +37,8 @@ def _sympify_string(math_string):
     # sympify doesn't recognize LN as ln()
     expr_string = \
         re.sub(r'(?<!\w)LN(?!\w)', 'ln', expr_string, flags=re.IGNORECASE)
+    expr_string = \
+        re.sub(r'(?<!\w)LOG(?!\w)', 'log', expr_string, flags=re.IGNORECASE)
     expr_string = \
         re.sub(r'(?<!\w)EXP(?!\w)', 'exp', expr_string,
                flags=re.IGNORECASE)
@@ -481,8 +483,8 @@ class TCPrinter(StrPrinter):
 
     def _print_Function(self, expr):
         func_translations = {'log': 'ln', 'exp': 'exp'}
-        if expr.func.__name__ in func_translations:
-            return func_translations[expr.func.__name__] + "(%s)" % self.stringify(expr.args, ", ")
+        if expr.func.__name__.lower() in func_translations:
+            return func_translations[expr.func.__name__.lower()] + "(%s)" % self.stringify(expr.args, ", ")
         else:
             raise TypeError("Unable to represent function: %s" %
                              expr.func.__name__)
