@@ -4,12 +4,13 @@ calculations under specified conditions.
 """
 from __future__ import division
 import copy
-from sympy import log, Abs, Add, Mul, Piecewise, Pow, S, sin, Symbol
+from sympy import exp, log, Abs, Add, Mul, Piecewise, Pow, S, sin, Symbol, zoo, oo
 from tinydb import where
 import pycalphad.variables as v
 from pycalphad.core.constants import MIN_SITE_FRACTION
 from pycalphad.log import logger
 import numpy as np
+from collections import OrderedDict
 
 # Maximum number of levels deep we check for symbols that are functions of
 # other symbols
@@ -26,11 +27,11 @@ class Model(object):
 
     Parameters
     ----------
-    dbf : Database
+    dbe : Database
         Database containing the relevant parameters.
     comps : list
         Names of components to consider in the calculation.
-    phase : list
+    phase_name : str
         Name of phase model to build.
     parameters : dict
         Optional dictionary of parameters to be substituted in the model.
@@ -44,25 +45,36 @@ class Model(object):
     --------
     None yet.
     """
-    def __init__(self, dbe, comps, phase, parameters=None):
+    # We only use the contributions attribute in build_phase.
+    # Users should not access it later since subclasses can override build_phase
+    # and make self.models inconsistent with contributions.
+    # Note that we include atomic ordering last since it uses self.models
+    # to figure out its contribution.
+    contributions = [('ref', 'reference_energy'), ('idmix', 'ideal_mixing_energy'),
+                     ('xsmix', 'excess_mixing_energy'), ('mag', 'magnetic_energy'),
+                     ('2st', 'twostate_energy'), ('ein', 'einstein_energy'),
+                     ('ord', 'atomic_ordering_energy')]
+
+    def __init__(self, dbe, comps, phase_name, parameters=None):
         # Constrain possible components to those within phase's d.o.f
         possible_comps = set([x.upper() for x in comps])
         self.components = set()
         self.constituents = []
-        self.phase_name = phase.upper()
-        self.site_ratios = dbe.phases[phase.upper()].sublattices
-        for sublattice in dbe.phases[phase.upper()].constituents:
+        self.phase_name = phase_name.upper()
+        phase = dbe.phases[self.phase_name]
+        self.site_ratios = phase.sublattices
+        for sublattice in phase.constituents:
             self.components |= set(sublattice).intersection(possible_comps)
-        logger.debug('Model of %s has components %s', phase, self.components)
+        logger.debug('Model of %s has components %s', self.phase_name, self.components)
         # Verify that this phase is still possible to build
-        for sublattice in dbe.phases[phase.upper()].constituents:
+        for sublattice in phase.constituents:
             if len(set(sublattice).intersection(self.components)) == 0:
                 # None of the components in a sublattice are active
                 # We cannot build a model of this phase
                 raise DofError(
                     '{0}: Sublattice {1} of {2} has no components in {3}' \
-                    .format(phase.upper(), sublattice,
-                            dbe.phases[phase.upper()].constituents,
+                    .format(self.phase_name, sublattice,
+                            phase.constituents,
                             self.components))
             self.constituents.append(set(sublattice).intersection(self.components))
 
@@ -73,8 +85,8 @@ class Model(object):
         if parameters is not None:
             symbols.update([(Symbol(s), val) for s, val in parameters.items()])
 
-        self.models = dict()
-        self.build_phase(dbe, phase.upper(), symbols, dbe.search)
+        self.models = OrderedDict()
+        self.build_phase(dbe)
         self.site_fractions = sorted(self.ast.atoms(v.SiteFraction), key=str)
 
         for name, value in self.models.items():
@@ -180,6 +192,7 @@ class Model(object):
 
     # Note: In order-disorder phases, TC will always be the *disordered* value of TC
     curie_temperature = TC = S.Zero
+    neel_temperature = NT = S.Zero
 
     #pylint: disable=C0103
     # These are standard abbreviations from Thermo-Calc for these quantities
@@ -195,31 +208,23 @@ class Model(object):
     mixing_heat_capacity = CPM_MIX = \
         property(lambda self: -v.T*self.GM_MIX.diff(v.T, v.T))
 
-    def build_phase(self, dbe, phase_name, symbols, param_search):
+    def build_phase(self, dbe):
         """
-        Apply phase's model hints to build a master SymPy object.
-        """
-        phase = dbe.phases[phase_name]
-        self.models['ref'] = self.reference_energy(phase, param_search)
-        self.models['idmix'] = self.ideal_mixing_energy(phase, param_search)
-        self.models['xsmix'] = self.excess_mixing_energy(phase, param_search)
-        self.models['mag'] = self.magnetic_energy(phase, param_search)
+        Generate the symbolic form of all the contributions to this phase.
 
-        # Next, we handle atomic ordering
-        # NOTE: We need to add this one last since it uses self.models
-        # to figure out the contribution.
-        # It will also MODIFY self.models
-        ordered_phase_name = None
-        disordered_phase_name = None
-        try:
-            ordered_phase_name = phase.model_hints['ordered_phase']
-            disordered_phase_name = phase.model_hints['disordered_phase']
-        except KeyError:
-            pass
-        if ordered_phase_name == phase_name:
-            self.models['ord'] = self.atomic_ordering_energy(dbe,
-                                                             disordered_phase_name,
-                                                             ordered_phase_name)
+        Parameters
+        ----------
+        dbe : Database
+        """
+        contrib_vals = list(OrderedDict(self.__class__.contributions).values())
+        if 'atomic_ordering_energy' in contrib_vals:
+            if contrib_vals.index('atomic_ordering_energy') != (len(contrib_vals) - 1):
+                # Check for a common mistake in custom models
+                # Users that need to override this behavior should override build_phase
+                raise ValueError('\'atomic_ordering_energy\' must be the final contribution')
+        self.models.clear()
+        for key, value in self.__class__.contributions:
+            self.models[key] = S(getattr(self, value)(dbe))
 
     def _purity_test(self, constituent_array):
         """
@@ -389,26 +394,29 @@ class Model(object):
             rk_terms.append(mixing_term * param['parameter'])
         return Add(*rk_terms)
 
-    def reference_energy(self, phase, param_search):
+    def reference_energy(self, dbe):
         """
         Returns the weighted average of the endmember energies
         in symbolic form.
         """
         pure_param_query = (
-            (where('phase_name') == phase.name) & \
+            (where('phase_name') == self.phase_name) & \
             (where('parameter_order') == 0) & \
             (where('parameter_type') == "G") & \
             (where('constituent_array').test(self._purity_test))
         )
+        phase = dbe.phases[self.phase_name]
+        param_search = dbe.search
         pure_energy_term = self.redlich_kister_sum(phase, param_search,
                                                    pure_param_query)
         return pure_energy_term / self._site_ratio_normalization(phase)
 
-    def ideal_mixing_energy(self, phase, param_search):
+    def ideal_mixing_energy(self, dbe):
         #pylint: disable=W0613
         """
         Returns the ideal mixing energy in symbolic form.
         """
+        phase = dbe.phases[self.phase_name]
         # Normalize site ratios
         site_ratio_normalization = self._site_ratio_normalization(phase)
         site_ratios = phase.sublattices
@@ -429,7 +437,7 @@ class Model(object):
         ideal_mixing_term *= (v.R * v.T)
         return ideal_mixing_term
 
-    def excess_mixing_energy(self, phase, param_search):
+    def excess_mixing_energy(self, dbe):
         """
         Build the binary, ternary and higher order interaction term
         Here we use Redlich-Kister polynomial basis by default
@@ -437,8 +445,10 @@ class Model(object):
         Replace y_i -> y_i + (1 - sum(y involved in parameter)) / m,
         where m is the arity of the interaction parameter
         """
+        phase = dbe.phases[self.phase_name]
+        param_search = dbe.search
         param_query = (
-            (where('phase_name') == phase.name) & \
+            (where('phase_name') == self.phase_name) & \
                 ((where('parameter_type') == 'G') |
                  (where('parameter_type') == 'L')) & \
                 (where('constituent_array').test(self._interaction_test))
@@ -446,13 +456,15 @@ class Model(object):
         excess_term = self.redlich_kister_sum(phase, param_search, param_query)
         return excess_term / self._site_ratio_normalization(phase)
 
-    def magnetic_energy(self, phase, param_search):
+    def magnetic_energy(self, dbe):
         #pylint: disable=C0103, R0914
         """
         Return the energy from magnetic ordering in symbolic form.
         The implemented model is the Inden-Hillert-Jarl formulation.
-        The approach follows from the background section of W. Xiong, 2011.
+        The approach follows from the background of W. Xiong et al, Calphad, 2012.
         """
+        phase = dbe.phases[self.phase_name]
+        param_search = dbe.search
         self.TC = self.curie_temperature = S.Zero
         if 'ihj_magnetic_structure_factor' not in phase.model_hints:
             return S.Zero
@@ -463,6 +475,10 @@ class Model(object):
         # define basic variables
         afm_factor = phase.model_hints['ihj_magnetic_afm_factor']
 
+        if afm_factor == 0:
+            # Apply improved magnetic model which does not use AFM / Weiss factor
+            return self.xiong_magnetic_energy(dbe)
+
         bm_param_query = (
             (where('phase_name') == phase.name) & \
             (where('parameter_type') == 'BMAGN') & \
@@ -510,6 +526,124 @@ class Model(object):
         return v.R * v.T * log(beta+1) * \
             g_term / site_ratio_normalization
 
+    def xiong_magnetic_energy(self, dbe):
+        """
+        Return the energy from magnetic ordering in symbolic form.
+        The approach follows W. Xiong et al, Calphad, 2012.
+        """
+        phase = dbe.phases[self.phase_name]
+        param_search = dbe.search
+        self.TC = self.curie_temperature = S.Zero
+        if 'ihj_magnetic_structure_factor' not in phase.model_hints:
+            return S.Zero
+        if 'ihj_magnetic_afm_factor' not in phase.model_hints:
+            return S.Zero
+
+        site_ratio_normalization = self._site_ratio_normalization(phase)
+        # define basic variables
+        afm_factor = phase.model_hints['ihj_magnetic_afm_factor']
+
+        if afm_factor != 0:
+            raise ValueError('Xiong model called with nonzero AFM / Weiss factor')
+
+        nt_param_query = (
+            (where('phase_name') == phase.name) & \
+            (where('parameter_type') == 'NT') & \
+            (where('constituent_array').test(self._array_validity))
+        )
+
+        bm_param_query = (
+            (where('phase_name') == phase.name) & \
+            (where('parameter_type') == 'BMAGN') & \
+            (where('constituent_array').test(self._array_validity))
+        )
+        tc_param_query = (
+            (where('phase_name') == phase.name) & \
+            (where('parameter_type') == 'TC') & \
+            (where('constituent_array').test(self._array_validity))
+        )
+
+        mean_magnetic_moment = \
+            self.redlich_kister_sum(phase, param_search, bm_param_query)
+        beta = mean_magnetic_moment
+
+        curie_temp = \
+            self.redlich_kister_sum(phase, param_search, tc_param_query)
+        neel_temp = \
+            self.redlich_kister_sum(phase, param_search, nt_param_query)
+
+        self.TC = self.curie_temperature = curie_temp.xreplace(dbe.symbols)
+        self.NT = self.neel_temperature = neel_temp.xreplace(dbe.symbols)
+
+        tau_curie = v.T / curie_temp
+        tau_curie = tau_curie.xreplace({zoo: 1.0e10})
+        tau_neel = v.T / neel_temp
+        tau_neel = tau_neel.xreplace({zoo: 1.0e10})
+
+        # define model parameters
+        p = phase.model_hints['ihj_magnetic_structure_factor']
+        D = 0.33471979 + 0.49649686*(1/p - 1)
+        sub_tau_curie = 1 - (1/D) * ((0.38438376/p)*(tau_curie**(-1)) + 0.63570895*(1/p - 1) \
+            * ((tau_curie**3)/6 + (tau_curie**9)/135 + (tau_curie**15)/600) + (tau_curie**21)/1617
+                              )
+        sub_tau_neel = 1 - (1/D) * ((0.38438376/p)*(tau_neel**(-1)) + 0.63570895*(1/p - 1) \
+            * ((tau_neel**3)/6 + (tau_neel**9)/135 + (tau_neel**15)/600) + (tau_neel**21)/1617
+                              )
+        super_tau_curie = -(1/D) * ((tau_curie**-7)/21 + (tau_curie**-21)/630 + (tau_curie**-35)/2975 + (tau_curie**-49)/8232)
+        super_tau_neel = -(1/D) * ((tau_neel**-7)/21 + (tau_neel**-21)/630 + (tau_neel**-35)/2975 + (tau_neel**-49)/8232)
+
+        expr_cond_pairs_curie = [(0, tau_curie <= 0),
+                                 (super_tau_curie, tau_curie > 1),
+                                 (sub_tau_curie, True)
+                                ]
+        expr_cond_pairs_neel = [(0, tau_neel <= 0),
+                                (super_tau_neel, tau_neel > 1),
+                                (sub_tau_neel, True)
+                               ]
+        g_term = Piecewise(*expr_cond_pairs_curie) + Piecewise(*expr_cond_pairs_neel)
+
+        return v.R * v.T * log(beta+1) * \
+            g_term / site_ratio_normalization
+
+    def twostate_energy(self, dbe):
+        """
+        Return the energy from liquid-amorphous two-state model.
+        """
+        phase = dbe.phases[self.phase_name]
+        param_search = dbe.search
+        site_ratio_normalization = self._site_ratio_normalization(phase)
+        gd_param_query = (
+            (where('phase_name') == phase.name) & \
+            (where('parameter_type') == 'GD') & \
+            (where('constituent_array').test(self._array_validity))
+        )
+        gd = self.redlich_kister_sum(phase, param_search, gd_param_query)
+        if gd == S.Zero:
+            return S.Zero
+        return -v.R * v.T * log(1 + exp(-gd / (v.R * v.T))) / site_ratio_normalization
+
+    def einstein_energy(self, dbe):
+        """
+        Return the energy based on the Einstein model.
+        Note that THETA parameters are actually LN(THETA).
+        All Redlich-Kister summation is done in log-space,
+        then exp() is called on the result.
+        """
+        phase = dbe.phases[self.phase_name]
+        param_search = dbe.search
+        theta_param_query = (
+            (where('phase_name') == phase.name) & \
+            (where('parameter_type') == 'THETA') & \
+            (where('constituent_array').test(self._array_validity))
+        )
+        lntheta = self.redlich_kister_sum(phase, param_search, theta_param_query)
+        theta = exp(lntheta)
+        if lntheta != 0:
+            result = 1.5*v.R*theta + 3*v.R*v.T*log(1-exp(-theta/v.T))
+        else:
+            result = 0
+        return result / self._site_ratio_normalization(phase)
+
     @staticmethod
     def mole_fraction(species_name, phase_name, constituent_array,
                       site_ratios):
@@ -547,12 +681,16 @@ class Model(object):
 
         return numerator / site_ratio_normalization
 
-    def atomic_ordering_energy(self, dbe, disordered_phase_name,
-                               ordered_phase_name):
+    def atomic_ordering_energy(self, dbe):
         """
         Return the atomic ordering contribution in symbolic form.
         Description follows Servant and Ansara, Calphad, 2001.
         """
+        phase = dbe.phases[self.phase_name]
+        ordered_phase_name = phase.model_hints.get('ordered_phase', None)
+        disordered_phase_name = phase.model_hints.get('disordered_phase', None)
+        if phase.name != ordered_phase_name:
+            return S.Zero
         disordered_model = self.__class__(dbe, self.components,
                                           disordered_phase_name)
         constituents = [sorted(set(c).intersection(self.components)) \
